@@ -5,29 +5,28 @@
 #' @rdname coef.abess
 #'
 #' @param object An "\code{abess}" project.
-#' @param support.size An integer vector specifies 
-#' the coefficient fitted at given \code{support.size}. 
-#' If \code{support.size = NULL}, then all coefficients would be returned. 
+#' @param support.size An integer vector specifies
+#' the coefficient fitted at given \code{support.size}.
+#' If \code{support.size = NULL}, then all coefficients would be returned.
 #' Default: \code{support.size = NULL}.
-#' 
+#'
 #' @param sparse A logical value, specifying whether the coefficients should be
 #' presented as sparse matrix or not. Default: \code{sparse = TRUE}.
 #' @param ... Other arguments.
-#' 
-#' @return A coefficient matrix when fitting an univariate model including gaussian, binomial, poisson, and cox; 
-#' otherwise, a list containing coefficient matrices. 
-#' For a coefficient matrix, each row is a variable, and each column is a support size. 
-#' 
+#'
+#' @return A coefficient matrix when fitting an univariate model including gaussian, binomial, poisson, and cox;
+#' otherwise, a list containing coefficient matrices.
+#' For a coefficient matrix, each row is a variable, and each column is a support size.
+#'
 #' @inherit abess.default seealso
-#' 
+#'
 #' @method coef abess
-#' 
+#'
 #' @export
 #'
-coef.abess <- function(object, 
-                       support.size = NULL, 
-                       sparse = TRUE, ...)
-{
+coef.abess <- function(object,
+                       support.size = NULL,
+                       sparse = TRUE, ...) {
   supp_size_index <- NULL
   if (!is.null(support.size)) {
     supp_size_index <- match_support_size(object, support.size)
@@ -36,12 +35,14 @@ coef.abess <- function(object,
   }
   stopifnot(is.logical(sparse))
   multi_y <- object[["family"]] %in% MULTIVARIATE_RESPONSE
-  
+
   if (multi_y) {
     coef <- list()
     for (i in 1:length(supp_size_index)) {
-      coef[[i]] <- combine_beta_intercept(object[["beta"]][[supp_size_index[i]]], 
-                                          object[["intercept"]][[supp_size_index[i]]])
+      coef[[i]] <- combine_beta_intercept(
+        object[["beta"]][[supp_size_index[i]]],
+        object[["intercept"]][[supp_size_index[i]]]
+      )
     }
   } else {
     coef <- combine_beta_intercept(object[["beta"]], object[["intercept"]])
@@ -49,7 +50,7 @@ coef.abess <- function(object,
       coef <- coef[, supp_size_index, drop = FALSE]
     }
   }
-  
+
   if (!sparse) {
     if (multi_y) {
       coef <- lapply(coef, as.matrix)
@@ -57,7 +58,7 @@ coef.abess <- function(object,
       coef <- as.matrix(coef)
     }
   }
-  
+
   coef
 }
 
@@ -68,4 +69,4 @@ combine_beta_intercept <- function(beta_mat, intercept_vec) {
   rownames(beta0) <- "(intercept)"
   coef <- methods::rbind2(beta0, coef)
   coef
-}
\ No newline at end of file
+}
