@@ -231,6 +231,10 @@ class Metric {
         } else if (ic_type == 4) {
             return loss +
                    this->ic_coef * (log(double(train_n)) + 2 * log(double(N))) * algorithm->get_effective_number();
+        } else if (ic_type == 5) {
+            return train_n *
+                       (algorithm->get_train_loss() - algorithm->lambda_level * algorithm->beta.cwiseAbs2().sum()) +
+                   this->ic_coef * log(double(N)) * log(log(double(train_n))) * algorithm->get_effective_number();
         } else
             return 0;
     };
