@@ -8,9 +8,10 @@
 #ifndef DISABLE_OPENMP
 #ifndef R_BUILD
 // use pragma message instead of warning
-#pragma message("Warning: OpenMP is not available, "                 \
-                "project will be compiled into single-thread code. " \
-                "Use OpenMP-enabled compiler to get benefit of multi-threading.")
+#pragma message(                                         \
+    "Warning: OpenMP is not available, "                 \
+    "project will be compiled into single-thread code. " \
+    "Use OpenMP-enabled compiler to get benefit of multi-threading.")
 #endif
 #endif
 inline int omp_get_thread_num() { return 0; }
@@ -20,4 +21,4 @@ inline void omp_set_num_threads(int nthread) {}
 inline void omp_set_dynamic(int flag) {}
 #endif
 
-#endif // SRC_ABESSOPENMP_H
+#endif  // SRC_ABESSOPENMP_H
